from rest_framework import serializers
from ..sparrow_aftersale.models import Afs

from .models import Distribute, Distributeline,  FootPrint,DistributelineEcard, ExpressRelaBack
from ..sparrow_orders.constants import OrderTypes
from ..sparrow_orders.models import ShippingAddress
from ..sparrow_shipping.models import NewExpressOrder


class ShippingAddressSerializer(serializers.ModelSerializer):
    '''收件地址，修改地址的时候也用这个'''

    class Meta:
        model = ShippingAddress
        fields = '__all__'

class NewExpressOrderSerializer(serializers.ModelSerializer):

    pay_card_number_name = serializers.CharField(max_length=256,required=False)

    class Meta:
        model = NewExpressOrder
        fields = '__all__'

class LanyueLightDistributeSerializer(serializers.ModelSerializer):
    '''整个轻量发货单，包括发货单表头和表体'''
    # distributelines = DistributelineSerializer(many=True, required=False)
    # available_event_list = serializers.ListField(required=False, child=serializers.CharField(required=False))
    # shipping_address = ShippingAddressSerializer(required=False, read_only=True)
    # express_order = NewExpressOrderSerializer(required=False, read_only=True)
    # cusservice_footprints = FootPrintSerializer(many=True, required=False)
    # ex_status_for_b = serializers.IntegerField(required=False)
    class Meta:
        model = Distribute
        fields = '__all__'

class DistributelineEcardSerializer(serializers.ModelSerializer):
    '''发货单Line E卡信息'''
    class Meta:
        model = DistributelineEcard
        fields = '__all__'

class DistributelineSerializer(serializers.ModelSerializer):
    '''发货单表体'''

    title = serializers.SerializerMethodField()
    afs_version = serializers.SerializerMethodField()
    ecard_info = DistributelineEcardSerializer()

    def get_title(self, obj):
        title = obj.title
        order_type = obj.order_type
        if order_type == OrderTypes.POINTS_MALL:  # 积分商城
            title = "【积分商城】" + title
        return title

    def get_afs_version(self, obj):
        if obj.aftersale_id is not None:
            afs = Afs.objects.filter(id=obj.aftersale_id).first()
            if afs:
                return afs.afs_version
        return None

    class Meta:
        model = Distributeline
        fields = '__all__'


class LanyueWholeDistributeSerializer(serializers.ModelSerializer):
    '''整个发货单，包括发货单表头和表体'''
    distributelines = DistributelineSerializer(many=True, required=False)
    # available_event_list = serializers.ListField(required=False, child=serializers.CharField(required=False))
    shipping_address = ShippingAddressSerializer(required=False, read_only=True)
    express_order = NewExpressOrderSerializer(required=False, read_only=True)

    # cusservice_footprints = FootPrintSerializer(many=True, required=False)
    class Meta:
        model = Distribute
        fields = '__all__'

class FootPrintSerializer(serializers.ModelSerializer):
    '''整个发货单，包括发货单表头和表体'''

    class Meta:
        model = FootPrint
        fields = '__all__'

class ExpressRelaBackSerializer(serializers.ModelSerializer):
    class Meta:
        model = ExpressRelaBack
        fields = '__all__'

class CSWholeDistributeDetailSerializer(serializers.ModelSerializer):
    '''整个发货单，包括发货单表头和表体'''
    distributelines = DistributelineSerializer(many=True, required=False)
    available_event_list = serializers.ListField(required=False, child=serializers.CharField(required=False))
    shipping_address = ShippingAddressSerializer(required=False, read_only=True)
    express_order = NewExpressOrderSerializer(required=False, read_only=True)
    cusservice_footprints = FootPrintSerializer(many=True, required=False)
    express_routes = serializers.ListField(required=False)
    express_history = ExpressRelaBackSerializer(many=True, required=False)
    class Meta:
        model = Distribute
        fields = '__all__'


import logging
from io import BytesIO
import xlsxwriter

logger = logging.getLogger(__name__)


# write_to_excel：把data数据写入到excel中
def write_to_excel(named_result, sheet_name):
    err_msg = ""
    output = BytesIO()
    workbook = xlsxwriter.Workbook(output)
    ws = workbook.add_worksheet(sheet_name)
    # 列名加粗显示
    bold = workbook.add_format({'bold': True})
    try:
        # 准备提取列明
        columns = []
        if named_result and len(named_result) > 0:
            if len(named_result[0]) > 0:
                for k in named_result[0][0]._fields:
                    columns.append(k)

                if len(columns) > 0:
                    ws.write(0, 0, "序号", bold)
                    ws.write_row(0, 1, columns, bold)
                    row_idx = 1

                    for d in named_result[0]:
                        ws.write(row_idx, 0, row_idx)
                        ws.write_row(row_idx, 1, d)
                        row_idx += 1
            elif len(named_result) > 1 and len(named_result[1]) > 0:
                ws.write_string(0, 0, named_result[1])
    except Exception as e:
        err_msg = "保存excel处理数据时时失败,原因: {be_str}".format(be_str=str(e))
        ws.write_string(0, 0, err_msg)
        logger.error(err_msg)
    finally:
        workbook.close()
    xlsx_data = output.getvalue()
    return xlsx_data, err_msg



'''
查询入参
'''


class InQueryParam(object):
    ''' 查询入参 '''

    def __init__(self, in_param_key, in_param_value):
        self.__in_param_key = in_param_key
        self.__in_param_value = in_param_value

    @property
    def in_param_key(self):
        return self.__in_param_key

    @property
    def in_param_value(self):
        return self.__in_param_value



from ..sparrow_distribute.models import Distributeline, DistributelineEcard, ExpressRela
from ..sparrow_orders.constants import DeliverTimeType, PayMethod
from ..sparrow_orders.models import ShippingAddress
from ..sparrow_shipping.models import NewExpressOrder

from .serializers import ShippingAddressSerializer, NewExpressOrderSerializer, \
    DistributelineSerializer


def decorator_dislines_in_distribute_list(serializer_data, serializer_cls=DistributelineSerializer):
    '''
    发货单行
    '''
    dis_ids = [distribute_dict['id'] for distribute_dict in serializer_data]
    disline_obj_list = Distributeline.objects.filter(distribute_id__in=dis_ids)

    disline_ids = [disline_obj.id for disline_obj in disline_obj_list]

    disline_ecard_list = DistributelineEcard.objects.filter(distribute_line_id__in=disline_ids)
    disline_id_2_disline_ecard_dict = {disline_ecard.distribute_line_id: disline_ecard for disline_ecard in
                                       disline_ecard_list}
    dis_id2lines_dict = {}
    for disline_obj in disline_obj_list:
        if disline_obj.id in disline_id_2_disline_ecard_dict:
            disline_obj.ecard_info = disline_id_2_disline_ecard_dict[disline_obj.id]
        else:
            disline_obj.ecard_info = None
        distribute_id = disline_obj.distribute_id
        if distribute_id not in dis_id2lines_dict:
            dis_id2lines_dict[distribute_id] = []
        dis_id2lines_dict[distribute_id].append(disline_obj)
    for distribute_dict in serializer_data:
        distribute_id = distribute_dict['id']
        disline_obj_list = dis_id2lines_dict.get(distribute_id, None)
        if not disline_obj_list:
            distribute_dict.update({'distributelines': []})
        else:
            disline_serializer = serializer_cls(instance=disline_obj_list, many=True, required=False)
            disline_serializer_data = disline_serializer.data
            distribute_dict.update({'distributelines': disline_serializer_data})


def decorator_expressorder_in_distribute_list(serializer_data, serializer_cls=NewExpressOrderSerializer):
    '''
    运单信息
    '''
    distribute_ids = [distribute_dict['id'] for distribute_dict in serializer_data]
    rela_obj_list = ExpressRela.objects.filter(distribute_id__in=distribute_ids)
    rela_obj_dict = {str(rela_obj.distribute_id): str(rela_obj.newexpressorder_id) for rela_obj in rela_obj_list}

    expressorder_id_list = [rela_obj.newexpressorder_id for rela_obj in rela_obj_list]
    expressorder_obj_list = NewExpressOrder.objects.filter(id__in=expressorder_id_list)
    expressorder_dict = {str(expressorder_obj.id): expressorder_obj for expressorder_obj in expressorder_obj_list}

    distribute_express_info_dict = {}
    for distribute_id in distribute_ids:
        distribute_id_key = str(distribute_id)
        if distribute_id_key not in rela_obj_dict:
            distribute_express_info_dict[distribute_id_key] = None
        else:
            newexpressorder_id = rela_obj_dict[distribute_id_key]
            newexpressorder_id_key = str(newexpressorder_id)
            if newexpressorder_id_key not in expressorder_dict:
                distribute_express_info_dict[distribute_id_key] = None
            else:
                expressorder_obj = expressorder_dict[newexpressorder_id_key]
                expressorder_obj_serializer = serializer_cls(instance=expressorder_obj, required=False)
                distribute_express_info_dict[distribute_id_key] = expressorder_obj_serializer.data

    for distribute_dict in serializer_data:
        distribute_id = distribute_dict['id']
        expressorder_obj_data = distribute_express_info_dict[distribute_id]
        distribute_dict.update({"express_order": expressorder_obj_data})


def decorator_shippingaddr_in_distribute_list(serializer_data, serializer_cls=ShippingAddressSerializer):
    '''
    快递地址
    '''
    shipping_address_id_list = [distribute_dict['shipping_address_id'] for distribute_dict in serializer_data]
    shipaddr_obj_list = ShippingAddress.objects.filter(id__in=shipping_address_id_list)
    shipaddrid_2_obj_dict = {str(shipaddr_obj.id): shipaddr_obj for shipaddr_obj in shipaddr_obj_list}
    for distribute_dict in serializer_data:
        shipping_address_id = distribute_dict['shipping_address_id']
        shipaddr_obj = shipaddrid_2_obj_dict.get(str(shipping_address_id), None)
        if not shipaddr_obj:
            distribute_dict.update({'shipping_address': None})
        else:
            shipaddr_obj_serializer = serializer_cls(instance=shipaddr_obj, required=False)
            distribute_dict.update({'shipping_address': shipaddr_obj_serializer.data})


def decorator_deliver_time_str_in_distribute_list(serializer_data):
    """ 对于分阶段支付订单, 对每个发货单行添加发货时间 """

    for distribute_data in serializer_data:
        if 'distributelines' not in distribute_data:
            continue

        if distribute_data["pay_method"] != PayMethod.TWICE:
            continue

        disline_datas = distribute_data['distributelines']

        for disline_data in disline_datas:

            deliver_time = disline_data.get('deliver_time')
            deliver_time_type = disline_data.get('deliver_time_type')
            deliver_relative_time = disline_data.get('deliver_relative_time')

            deliver_time_str = ""

            if deliver_time:  # 已支付尾款或固定发货时间
                deliver_time_show = str(deliver_time)
                deliver_time_show = deliver_time_show[0:10]
                deliver_time_str = "预计{}前发货".format(str(deliver_time_show))
            elif deliver_relative_time and deliver_time_type == DeliverTimeType.PAY_RELATIVE_TIME:  # 相对发货时间
                deliver_time_str = "付尾款后{}日内发货".format(int(float((deliver_relative_time))))

            disline_data["deliver_time_str"] = deliver_time_str


def decorator_product_group_in_distribute_list(serializer_data):
    '''
    商品库存状态
    '''

    # 遍历每一个订单
    for distribute_dict in serializer_data:

        if "distributelines" not in distribute_dict:
            continue
        distributelines = distribute_dict['distributelines']

        # 遍历每一个订单行
        productlines = {}
        for distributeline in distributelines:
            if "product_id" not in distributeline or not distributeline["product_id"]:
                continue

            if distributeline["line_id"] in productlines:
                # 旧
                _prod = productlines[distributeline["line_id"]]

                _prod["quantity"] = _prod["quantity"] + distributeline["quantity"]
                _prod["lines"].append(distributeline)
            else:
                # 新
                productlines[distributeline["line_id"]] = {
                    "product_id": distributeline["product_id"],
                    "quantity": distributeline["quantity"],
                    "title": distributeline["title"],
                    "is_gift": distributeline["is_gift"],
                    "sku_attr": distributeline["sku_attr"],
                    "shop_sku": distributeline["shop_sku"],
                    "lines": [distributeline],
                    "shop_income_price": distributeline["shop_income_price"],
                }

        # 不再返回
        distribute_dict['distributelines'] = None
        distribute_dict["products"] = productlines


def lanyue_valid_distributedetail(serializer_data, shop_id=0):
    """ 避免销售在地址栏直接输入其他专柜的发货单 """

    for distribute_data in serializer_data:
        if 'distributelines' not in distribute_data:
            continue

        disline_datas = distribute_data['distributelines']
        for disline_data in disline_datas:
            if disline_data["shop_id"] != shop_id:
                return False

    return True